# Core lyric_cix
# This file fetches html document from naver and parse it.
#
# ChangeLog:
# 09 Feb 2007: Modified from lyric_naver
#
require 'my_web'
require 'cgi'
require 'iconv'

module AmarokCix
  # Skip one utf-8 character from str
  def AmarokCix.skipOneCharacter(str)
    return nil if str == nil
    return "" if str == ""
    return str[1..-1] if str[0] < 128
    return "" if str.length == 1
    return str[2..-1] if str[0] < 224
    return "" if str.length == 2
    return str[3..-1] if str[0] < 240
    return "" if str.length == 3
    return str[4..-1]
  end

  # Convert query string from utf-8 to euc-kr which is cix's encoding
  def AmarokCix.getQueryString(query)
    query = CGI.unescape(query).delete('.')
    CGI.escape(Iconv.new('euc-kr', 'utf-8').iconv(query))
  rescue Iconv::IllegalSequence => e
    query = Iconv.new('utf-8', 'euc-kr').iconv(e.success) + "@" + skipOneCharacter(e.failed)
    retry
  end

  # Get candidates from fetched html
  def AmarokCix.getCandidates(artist, title)
    artistStr = getQueryString(artist+"@")
    titleStr = getQueryString(title+"@")
    url = "http://www.cix.co.kr/kr/search_lyrics_songlist_main.html?SearchSong=#{titleStr}&SearchSinger=#{artistStr}"
    body = MyWeb.fetchFromHttp(url)
    scanned = body.scan(/<TR BGCOLOR=[^>]*>\n\t<TD><span style='font-size:9pt;'><A HREF=([^ ]*) target=_top> ([^<&]*)[^\n]*\n\t<TD><span style='font-size:9pt;'><A HREF=[^ ]* target=_top> ([^<]*)<\/TD>\n/)
    scanned.each() do |entry|
      entry[0] = "http://www.cix.co.kr/kr/" + entry[0]
    end
    scanned.each() do |entry|
      entry[1] = Iconv.new('utf-8', 'euc-kr').iconv(entry[1])
      entry[2] = Iconv.new('utf-8', 'euc-kr').iconv(entry[2])
    end
    scanned
  end

  # parse lyric from fetched html
  def AmarokCix.parseLyric(body)
    matched = body.match(/<p ALIGN=center><font SIZE=\'4\' (FACE=\'\261\274\270\262\' )?color=black><br><b>([^<]*)<\/b><\/font><br><\/p>[^<]*<p align=right>[^<]*<font size=2>([^\n]*)\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*\n\t\t\t<br><font style='font-size:9pt;' color=black>\n\t\t\t([^\n]*)/)
    title, artist, lyric = [nil, nil, nil]
    if matched != nil
      matched = matched.captures
      title = Iconv.new('utf-8', 'euc-kr').iconv(matched[1])
      artist = Iconv.new('utf-8', 'euc-kr').iconv(matched[2])
      lyric = Iconv.new('utf-8', 'euc-kr').iconv(matched[3].gsub("<br>", "\n"))
    end
    return title, artist, lyric
  rescue Iconv::IllegalSequence
    return title, artist, lyric
  end

  # Get lyric from specified url
  def AmarokCix.getLyricFromUrl(url)
    body = MyWeb.fetchFromHttp(url)
    body != nil ? parseLyric(body) : [nil, nil, nil]
  end
end
