#!/usr/bin/env ruby
#
# amaroK Script for fetching song lyrics from http://www.cix.co.kr.
# Ported from amaroK's contextbrowser.cpp.
#
# (c) 2007 Jaehyun Yeom <only2sea@dreamwiz.com>
#
# License: GNU General Public License V2

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))
require 'amarok_cix'
require 'rexml/document'

@app_name = "Lyrics_CIX"

def showLyrics( lyrics )
  # Important, otherwise we might execute arbitrary nonsense in the DCOP call
  lyrics.gsub!( '"', "'" )
  lyrics.gsub!( '`', "'" )
  
  `dcop amarok contextbrowser showLyrics "#{lyrics}"`
end

def notFound()
  doc = REXML::Document.new( "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" )
  root = doc.add_element( "suggestions" )
  root.add_attribute( "page_url", @page_url )
  xml = ""
  doc.write( xml )
  showLyrics( xml )
end

def exportSuggestion(candidates)
  doc = REXML::Document.new( "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" )
  root = doc.add_element("suggestions")
  root.add_attribute("page_url", @page_url)
  candidates.each() do |candidate|
    suggestion = root.add_element("suggestion")
    suggestion.add_attribute("url",candidate[0])
    suggestion.add_attribute("title", candidate[1])
    suggestion.add_attribute("artist", candidate[2])
  end
  xml = ""
  doc.write(xml)
  showLyrics(xml)
end

def exportXml(artist, title, lyrics)
  doc = REXML::Document.new( "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" )
  root = doc.add_element("lyrics")

  root.add_attribute("page_url", @page_url)
  root.add_attribute("artist", artist)
  root.add_attribute("title", title)
  root.text = lyrics
  xml = ""
  doc.write(xml)
  showLyrics(xml)
end

def fetchLyricsByUrl(url)
  fetArtist, fetTitle, fetLyric = AmarokCix.getLyricFromUrl(url)
  if fetArtist == nil or fetTitle == nil or fetLyric == nil then
    notFound
  else
    exportXml(fetArtist, fetTitle, fetLyric)
  end
end

def fetchLyrics(artist, title)
  artist.tr!('_', ' ')
  title.tr!('_', ' ')
  candidates = AmarokCix.getCandidates(artist, title)
  if candidates.length == 0 then
    sqArtist = CGI.unescape(artist).delete(' ').delete('.').delete('-').gsub(/\([^\)]*\)/, "")
    sqTitle = CGI.unescape(title).delete(' ').delete('.').delete('-').gsub(/\([^\)]*\)/, "")
    if sqArtist != artist or sqTitle != title then
      fetchLyrics(sqArtist, sqTitle)
    else
      notFound
    end
  elsif candidates.length == 1 then
    fetchLyricsByUrl(candidates[0][0])
  else
    exportSuggestion(candidates)
  end
end

loop do
  message = gets().chomp()
  command = /[A-Za-z]*/.match( message ).to_s()

  case command
  when "configure"
    msg  = 'This script does not require any configuration.'
    `dcop amarok playlist popupMessage "#{msg}"`

  when "fetchLyrics"
    args = message.split()
    
    artist = args[1]
    title  = args[2]

    fetchLyrics(artist, title)

  when "fetchLyricsByUrl"
    url = message.split()[1]

    fetchLyricsByUrl(url)
  end
end

